//****************************************************************************
// @Module        Capture / Compare Unit 6 (CCU6)
// @Filename      CC6.C
// @Project       XC864_SCB.dav
//----------------------------------------------------------------------------
// @Controller    Infineon XC864L-1FR
//
// @Compiler      Keil
//
// @Codegenerator 1.1
//
// @Description   This file contains functions that use the CCU6 module.
//
//----------------------------------------------------------------------------
// @Date          8/15/2009 15:45:00
//
//****************************************************************************

// USER CODE BEGIN (CCU6_General,1)

// USER CODE END



//****************************************************************************
// @Project Includes
//****************************************************************************

#include "MAIN.H"

// USER CODE BEGIN (CCU6_General,2)
#include "CONFIG.H"

// USER CODE END


//****************************************************************************
// @Macros
//****************************************************************************

// USER CODE BEGIN (CCU6_General,3)

// USER CODE END


//****************************************************************************
// @Defines
//****************************************************************************

// USER CODE BEGIN (CCU6_General,4)

// USER CODE END


//****************************************************************************
// @Typedefs
//****************************************************************************

// USER CODE BEGIN (CCU6_General,5)

// USER CODE END


//****************************************************************************
// @Imported Global Variables
//****************************************************************************

// USER CODE BEGIN (CCU6_General,6)
extern  data	unsigned char 	g_slope;
extern	data	unsigned char 	g_state;

extern  data	unsigned char 	g_speed_counter;
extern  data	int  			g_duty_cyc;
extern 	bit 					gb_hs_error;
extern 	bit 					gb_ctrap;
extern 	bit	 					gb_time_out;
extern 	bit 					gb_On;
extern	bit 					gb_Off;
extern	bit						bituartRI;
extern  idata				int  			g_speed_CC60;
extern	data				long			g_pi_array_buf; 
extern  idata				unsigned int  	g_speed_ref ;
// USER CODE END


//****************************************************************************
// @Global Variables
//****************************************************************************

// USER CODE BEGIN (CCU6_General,7)
 data 	unsigned char 		task_counter;
 bit						bitTimeout;
 data  	unsigned int 		g_pi_controller_out;

// USER CODE END


//****************************************************************************
// @External Prototypes
//****************************************************************************

// USER CODE BEGIN (CCU6_General,8)
extern	int	SpeedCalculation();
extern void Ramp(void);
extern void Commutation(void);
extern void Channel_Select(void);
extern void MotorStop(void);
extern void Rampup(void);
extern void Scheduler(void);
extern  unsigned int PIControllerSpeed(long *buf_ptr,unsigned int reference,unsigned int act);
// USER CODE END


//****************************************************************************
// @Prototypes Of Local Functions
//****************************************************************************

// USER CODE BEGIN (CCU6_General,9)

// USER CODE END


//****************************************************************************
// @Function      void CC6_vInit(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the initialization function of the CCU6 function 
//                library. It is assumed that the SFRs used by this library 
//                are in their reset state. 
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          8/15/2009
//
//****************************************************************************

// USER CODE BEGIN (Init,1)

// USER CODE END

void CC6_vInit(void)
{
  // USER CODE BEGIN (Init,2)

  // USER CODE END

  SFR_PAGE(_cc1, noSST);         // switch to page 1

  ///  -----------------------------------------------------------------------
  ///  Configuration of Timer Control Register
  ///  -----------------------------------------------------------------------

  CCU6_TCTR0L    =  0x07;        // load CCU6 timer control register 0 low
  CCU6_TCTR0H    =  0x00;        // load CCU6 timer control register 0 high

  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 Timer 12:
  ///  -----------------------------------------------------------------------
  ///  - Timer T12 Input Clock Selected is 128
  ///  - The additional prescaler for T12 is disabled.
  ///  - timer 12 run bit is reset
  ///  - single shot mode is disabled
  ///  - timer 12 works in edge aligned mode
  ///  - interrupt on period match is enabled
  ///  - interrupt on one match is disabled

  CCU6_T12PRL    =  0xFF;        // load CCU6 T12 period register low
  CCU6_T12PRH    =  0xFF;        // load CCU6 T12 period register lhigh

  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 Timer 13:
  ///  -----------------------------------------------------------------------
  ///  - Timer T13 Input Clock Selected is 1
  ///  - The additional prescaler for T13 is disabled.
  ///  - timer 13 run bit is reset
  ///  - trigger control is disabled
  ///  - single shot mode is disabled
  ///  - interrupt on period match is enabled
  ///  - interrupt on compare match is disabled

  CCU6_T13PRL    =  0x35;        // load CCU6 T13 period register low
  CCU6_T13PRH    =  0x05;        // load CCU6 T13 period register high

  ///  -----------------------------------------------------------------------
  ///  Configuration of Dead-Time Control Register for Timer T12
  ///  -----------------------------------------------------------------------

  CCU6_T12DTCL   =  0x02;        // load CCU6 dead time control register low 
                                 // for timer T12

  CCU6_T12DTCH   =  0x01;        // load CCU6 dead time control register high 
                                 // for timer T12

  SFR_PAGE(_cc0, noSST);         // switch to page 0

  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 Channel 0:
  ///  -----------------------------------------------------------------------
  ///  - compare mode 3 is selected
  ///  - T12 modulation for output CC60 is disabled
  ///  - T13 modulation for output CC60 is disabled
  ///  - the output CC60 is set to the passive state
  ///  - the trap functionality of the pin CC60 is enabled
  ///  - the compare output CC60 drives passive level while CC60ST is '0'
  ///  - the passive level of the output CC60 is '1'
  ///  - T12 modulation for output COUT60 is disabled
  ///  - T13 modulation for output COUT60 is enabled
  ///  - the output COUT60 is set to the passive state
  ///  - the trap functionality of the pin COUT60 is enabled
  ///  - the compare output COUT60 drives passive level while CC60ST is '0'
  ///  - the passive level of the output COUT60 is '1'
  ///  - dead time generation is enabled

  ///  - generation interrupt on flag ICC60R is disabled
  ///  - generation interrupt on flag ICC60F is disabled

  CCU6_CC60SRL   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // low for channel 0
  CCU6_CC60SRH   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // high for channel 0


  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 Channel 1:
  ///  -----------------------------------------------------------------------
  ///  - compare mode 3 is selected
  ///  - T12 modulation for output CC61 is disabled
  ///  - T13 modulation for output CC61 is disabled
  ///  - the output CC61 is set to the passive state
  ///  - the trap functionality of the pin CC61 is enabled
  ///  - the compare output CC61 drives passive level while CC61ST is '0'
  ///  - the passive level of the output CC61 is '1'
  ///  - T12 modulation for output COUT61 is disabled
  ///  - T13 modulation for output COUT61 is enabled
  ///  - the output COUT61 is set to the passive state
  ///  - the trap functionality of the pin COUT61 is enabled
  ///  - the compare output COUT61 drives passive level while CC61ST is '0'
  ///  - the passive level of the output COUT61 is '1'
  ///  - dead time generation is disabled

  ///  - generation interrupt on flag ICC61R is disabled
  ///  - generation interrupt on flag ICC61F is disabled

  CCU6_CC61SRL   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // low for channel 1
  CCU6_CC61SRH   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // high for channel 1


  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 Channel 2:
  ///  -----------------------------------------------------------------------
  ///  - compare mode 3 is selected
  ///  - T12 modulation for output CC62 is disabled
  ///  - T13 modulation for output CC62 is disabled
  ///  - the output CC62 is set to the passive state
  ///  - the trap functionality of the pin CC62 is enabled
  ///  - the compare output CC62 drives passive level while CC62ST is '0'
  ///  - the passive level of the output CC62 is '1'
  ///  - T12 modulation for output COUT62 is disabled
  ///  - T13 modulation for output COUT62 is enabled
  ///  - the output COUT62 is set to the passive state
  ///  - the trap functionality of the pin COUT62 is enabled
  ///  - the compare output COUT62 drives passive level while CC62ST is '0'
  ///  - the passive level of the output COUT62 is '1'
  ///  - dead time generation is disabled

  ///  - generation interrupt on flag ICC62R is enabled
  ///  - generation interrupt on flag ICC62F is disabled

  CCU6_CC62SRL   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // low for channel 2
  CCU6_CC62SRH   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // high for channel 2


  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 Channel 3:
  ///  -----------------------------------------------------------------------
  ///  - T13 output is not inverted


  CCU6_CC63SRL   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // low for channel 3
  CCU6_CC63SRH   =  0x00;        // load CCU6 capture/compare shadow register 
                                 // high for channel 3


  ///  -----------------------------------------------------------------------
  ///  Configuration of Multi-Channel Mode Output Register
  ///  -----------------------------------------------------------------------

  CCU6_MCMOUTSL  =  0x00;        // load CCU6 multi channel mode output 
                                 // control register low
  CCU6_MCMOUTSH  =  0x00;        // load CCU6 multi channel mode output 
                                 // control register high

  SFR_PAGE(_cc2, noSST);         // switch to page 2

  ///  -----------------------------------------------------------------------
  ///  Configuration of Timer Control Register
  ///  -----------------------------------------------------------------------

  CCU6_TCTR2L    =  0x00;        // load CCU6 timer control register 2 low
  CCU6_TCTR2H    =  0x00;        // load CCU6 timer control register 2 high

  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 trap control:
  ///  -----------------------------------------------------------------------
  ///  - a trap can only be generated by SW by setting the bit TRPF
  ///  - the trap state is left immediately without any synchronization to 
  ///    T12 or T13
  ///  - the trap state can be left as soon as bit TRPF is reset by SW 
  ///    (according to TRPPEN, TRPM0 and TRPM1)
  ///  - trap interrupt is enabled

  CCU6_TRPCTRL   =  0x07;        // load CCU6 trap control register low
  CCU6_TRPCTRH   =  0x3F;        // load CCU6 trap control register high

  ///  -----------------------------------------------------------------------
  ///  Configuration of Multi Channel Mode:
  ///  -----------------------------------------------------------------------
  ///  - multi channel mode is enabled

  ///  - switching selection:
  ///  - transfer on T12 channel 1 compare match

  ///  - switching synchronization:
  ///  - synchronization on T13 zero-match


  CCU6_MODCTRL   =  0x80;        // load CCU6 modulation control register low
  CCU6_MODCTRH   =  0x2A;        // load CCU6 modulation control register high

  ///  -----------------------------------------------------------------------
  ///  Configuration of Multi_Channel Mode Control Register
  ///  -----------------------------------------------------------------------

  CCU6_MCMCTR    =  0x14;        // load CCU6 multi channel mode control 
                                 // register 

  ///  -----------------------------------------------------------------------
  ///  Configuration of T12 Capture/Compare Mode Select Register
  ///  -----------------------------------------------------------------------

  CCU6_T12MSELL  =  0x33;        // load CCU6 T12 campture/compare mode 
                                 // select register low

  CCU6_T12MSELH  =  0x03;        // load CCU6 T12 campture/compare mode 
                                 // select register high
  ///  -----------------------------------------------------------------------
  ///  Configuration of Passive State Level Register
  ///  -----------------------------------------------------------------------

  CCU6_PSLR      =  0x3F;        // load CCU6 passive state level register low

  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 interrupt control:
  ///  -----------------------------------------------------------------------
  ///  - for channel 0 interrupts is node I3 selected
  ///  - for channel 1 interrupts is node I3 selected
  ///  - for channel 2 interrupts is node I2 selected
  ///  - for correct hall event interrupt is node I3 selected
  ///  - for error interrupts is node I1 selected
  ///  - for T12 interrupts is node I1 selected
  ///  - for T13 interrupts is node I3 selected

  CCU6_INPL      =  0xEF;        // load CCU6 capture/compare interrupt node 
                                 // pointer register low
  CCU6_INPH      =  0x35;        // load CCU6 capture/compare interrupt node 
                                 // pointer register high

  CCU6_IENL      =  0x90;        // load CCU6 capture/compare interrupt 
                                 // enable register low
  CCU6_IENH      =  0x06;        // load CCU6 capture/compare interrupt 
                                 // enable register high

  SFR_PAGE(_cc3, noSST);         // switch to page 3

  ///  -----------------------------------------------------------------------
  ///  Configuration of Compare State Register
  ///  -----------------------------------------------------------------------

  CCU6_CMPSTATH  =  0x00;        // load CCU6 compare status register high

  ///  -----------------------------------------------------------------------
  ///  Configuration of CCU6 module input signals:
  ///  -----------------------------------------------------------------------
  ///  - signal CC60_0 is used as output
  ///  - signal CC61_1 is used as output
  ///  - signal CC62_1 is used as output
  ///  - signal CTRAP#_2 is used

  CCU6_PISEL0L   =  0x80;        // load CCU6 Port Input Select Register 0 Low

  ///  - signal CCPOS0 is not used
  ///  - signal CCPOS1 is not used
  ///  - signal CCPOS2 is not used
  ///  - signal T12HR is not used

  CCU6_PISEL0H   =  0x3F;        // load CCU6 Port Input Select Register 0 
                                 // High

  ///  - signal T13HR is not used

  CCU6_PISEL2    =  0x00;        // load CCU6 Port Input Select Register 2

  ///  Pin P0.0 is used as CC61_1 Output 
  ///  Pin P0.1 is used as COUT61_1 Output 
  ///  Pin P0.4 is used as CC62_1 Output 
  ///  Pin P0.5 is used as COUT62_1 Output 

  SFR_PAGE(_pp2, noSST);         // switch to page 2
  P0_ALTSEL0      &= ~(ubyte) 0x33;    //  set AltSel0 
  P0_ALTSEL1      |= 0x33;    //  set AltSel1

  SFR_PAGE(_pp0, noSST);         // switch to page 0
  P0_DIR          |= 0x33;    //  set Direction as Output

  ///  Pin P3.0 is used as CC60_0 Output 
  ///  Pin P3.1 is used as COUT60_0 Output 

  SFR_PAGE(_pp2, noSST);         // switch to page 2
  P3_ALTSEL0      |= 0x03;    //  set AltSel0 
  P3_ALTSEL1      &= ~(ubyte)0x03;    //  set AltSel1

  SFR_PAGE(_pp0, noSST);         // switch to page 0
  P3_DIR          |= 0x03;    //  set Direction as Output

  ///  -----------------------------------------------------------------------
  ///  Configuration of the used CCU6 Channels Interrupts:
  ///  -----------------------------------------------------------------------
  ///  - capture/compare interrupt node 0 is disabled
  ///  - capture/compare interrupt node 1 is enabled
  ///  - capture/compare interrupt node 2 is enabled
  ///  - capture/compare interrupt node 3 is enabled

  IEN1          |=  0xE0;        // load interrupt enable register 2


  SFR_PAGE(_cc0, noSST);         // switch to page 0

  ///  -----------------------------------------------------------------------
  ///  Timer Control Register
  ///  -----------------------------------------------------------------------
  ///  -  enable shadow transfer to T12 and T13

  CCU6_TCTR4L    =  0x40;        // load CCU6 timer control register 4 low

  CCU6_TCTR4H    =  0x40;        // load CCU6 timer control register 4 high

  // USER CODE BEGIN (Init,3)

  // USER CODE END

} //  End of function CC6_vInit

//****************************************************************************
// @Function      void CC6_viNodeI1(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the interrupt service routine for the CC6 node I1. 
//                If the content of the corresponding compare timer 
//                (configurable) equals the content of the capture/compare 
//                register or if a capture event occurs at the associated 
//                port pin, the interrupt request flag is set and an 
//                interrupt is triggered (only if enabled).
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          8/15/2009
//
//****************************************************************************

// USER CODE BEGIN (NodeI1,1)

// USER CODE END

void CC6_viNodeI1(void) interrupt CCU6_NodeI1_INT
{

  // USER CODE BEGIN (NodeI1,2)

  // USER CODE END

  SFR_PAGE(_cc3, SST2);          // switch to page 3

  if(CCU6_ISL & 0x80)   //if ISL_T12PM
  {
    //timer T12 period match detection

    SFR_PAGE(_cc0, noSST);       // switch to page 0

    CCU6_ISRL = 0x80;  //clear flag ISL_T12PM

    // USER CODE BEGIN (NodeI1,17)

	if(bitTimeout)
	{
	   	gb_time_out = 1;						   
   		MotorStop();
	}
	else
		bitTimeout=1;
    // USER CODE END

  }

  SFR_PAGE(_cc3, noSST);         // switch to page 3

  if(CCU6_ISH & 0x04)   //if ISH_TRPF
  {
    //trap detection

    SFR_PAGE(_cc0, noSST);       // switch to page 0

    CCU6_ISRH = 0x04;  //clear flag ISH_TRPF

    // USER CODE BEGIN (NodeI1,20)
		 gb_ctrap = 1;						   
    	 MotorStop();

    // USER CODE END

  }

  SFR_PAGE(_cc0, RST2);          // restore the old CCU page


} //  End of function CC6_viNodeI1

//****************************************************************************
// @Function      void CC6_viNodeI2(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the interrupt service routine for the CC6 node I2. 
//                If the content of the corresponding compare timer 
//                (configurable) equals the content of the capture/compare 
//                register or if a capture event occurs at the associated 
//                port pin, the interrupt request flag is set and an 
//                interrupt is triggered (only if enabled).
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          8/15/2009
//
//****************************************************************************

// USER CODE BEGIN (NodeI2,1)

// USER CODE END

void CC6_viNodeI2(void) interrupt CCU6_NodeI2_INT
{

  // USER CODE BEGIN (NodeI2,2)

  // USER CODE END

  SFR_PAGE(_cc3, SST2);          // switch to page 3

  if(CCU6_ISL & 0x10)   //if ISL_ICC62R
  {
    //capture, compare match rising edge detection an channel 2

    SFR_PAGE(_cc0, noSST);       // switch to page 0

    CCU6_ISRL = 0x10;  //clear flag ISL_ICC62R

    // USER CODE BEGIN (NodeI2,14)

	if (g_state == 3)											// if motor is running in steady state( state 3)...
	{
   		g_slope=g_slope-2;
   		g_speed_CC60 = SpeedCalculation();
	}
    // USER CODE END

  }

  SFR_PAGE(_cc0, RST2);          // restore the old CCU page


} //  End of function CC6_viNodeI2

//****************************************************************************
// @Function      void CC6_viNodeI3(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the interrupt service routine for the CC6 node I3. 
//                If the content of the corresponding compare timer 
//                (configurable) equals the content of the capture/compare 
//                register or if a capture event occurs at the associated 
//                port pin, the interrupt request flag is set and an 
//                interrupt is triggered (only if enabled).
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          8/15/2009
//
//****************************************************************************

// USER CODE BEGIN (NodeI3,1)

// USER CODE END

void CC6_viNodeI3(void) interrupt CCU6_NodeI3_INT
{

  // USER CODE BEGIN (NodeI3,2)

  // USER CODE END

  SFR_PAGE(_cc3, SST2);          // switch to page 3

  if(CCU6_ISH & 0x02)   //if ISH_T13PM
  {
    //timer T13 period match detection

    SFR_PAGE(_cc0, noSST);       // switch to page 0

    CCU6_ISRH = 0x02;  //clear flag ISH_T13PM

    // USER CODE BEGIN (NodeI3,19)
	if(g_state<3)
	{	

		Commutation();
	}
	else
	{
	   if(g_slope==1 ||g_slope==0)
		   Commutation();
	
	
	else
	{
		if(task_counter==0)
		{
		   g_pi_controller_out = PIControllerSpeed(&g_pi_array_buf,g_speed_ref,g_speed_CC60);  
		   g_duty_cyc = PULSE_PERIOD -g_pi_controller_out;
			if(g_duty_cyc < DUTY_CYCLE_MIN)
				g_duty_cyc = DUTY_CYCLE_MIN;
			else if(g_duty_cyc > DUTY_CYCLE_MAX)
				g_duty_cyc = DUTY_CYCLE_MAX;
	
			SFR_PAGE(_cc0, noSST);										// switch to page 0
			CCU6_CC63SRLH = g_duty_cyc;									// ....load it into the PWM duty cycle reg.
			CCU6_TCTR4H = 0x40;
			CCU6_TCTR4L = 0x00;
			task_counter = 1;
		}
		//-------------------------------------------------
		else if(task_counter==1)
		
		{	
		if(bituartRI)
		{
			bituartRI=0;
			Scheduler(); 
		}
			task_counter = 0;
		}
		else	
		{
			task_counter=0;
		
		}
	}
		Rampup();
	}

		Channel_Select();
    // USER CODE END

  }

  SFR_PAGE(_cc0, RST2);          // restore the old CCU page


} //  End of function CC6_viNodeI3


// USER CODE BEGIN (CCU6_General,10)

// USER CODE END

