//*****************************************************************************/
// @Module         PI Controller
// @Filename       PI_CTRL.C
// @Controller     XC88x Series
//
//
// @Description    This file contains functions for PI Control algorithms
//
//****************************************************************************

//****************************************************************************
// @Function  
// 
// int pi_controller64(long *pi_parameter,int reference,int actual) 
//
//----------------------------------------------------------------------------
// @Description  
//
//  PI-Controller
//      derived from transfer function G = kp + 1/(p*Ti)
//
//      e(k)   = reference - actual          T0 = 
//  yn(k+1)= yn(k)   + ki * e(k)             ki = T0/Ti
//  y(k+1) = yn(k+1) + kp * e(k) * 64
//----------------------------------------------------------------------------
// Computing time  42 CPU-cycle 
//
//----------------------------------------------------------------------------
// @Returnvalue   
//
//----------------------------------------------------------------------------
// @Parameters    
//
//--------------------------- Arguments -------------------------------------
//  reference : reference value
//  actual    : actual value
//
//  struct pi_parameter
//      { 
//      int yn;   Integral buffer
//      int kp;   Proportional Constant
//      int ki;   Integral Constant
//      int ymin; Limit value min
//      int ymax; Limit value max
//      };
//---------------------------------------------------------------------------
//  Register modified:
//
//  R1,R3,R4,R5,R6,R7,R8,R9,R10
//      
//----------------------------------------------------------------------------
//
//****************************************************************************
#include "main.H"
#include "CONFIG.H"

   
	   data		int					pierror;
extern data		long				g_pi_array_buf; 
extern idata	unsigned int 		speed_kp;
extern idata 	unsigned int 		speed_ki;

unsigned int PIControllerSpeed(long *buf_ptr,unsigned int reference,unsigned int act)
{
	
	int xdata y_ki,y_final;

	pierror	=	(reference-act );
	y_final	=	(int)(((long)pierror * speed_kp)>>9);
	
	*buf_ptr	=	*buf_ptr + (long)pierror * speed_ki; 	
	y_ki=(int)(*buf_ptr>>15);
	y_final = y_final + y_ki;

	if(y_final > PIC_SPEED_YMAX)
		y_final = PIC_SPEED_YMAX;
	else if(y_final < PIC_SPEED_YMIN)
		y_final = PIC_SPEED_YMIN;
	
	return(y_final);
	
}
//----------------------------------------------------------------



